#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/uio_driver.h>

#define DRIVER_AUTHOR "Dmitry Podkhvatilin <vatilin@gmail.com>"
#define DRIVER_DESC   "MCB-03PEM-PCI user I/O driver"

static int phys_addr_hi = 0x0;
module_param(phys_addr_hi, int, 0);
MODULE_PARM_DESC(phys_addr_hi, "High part (16 bits) of base physical address of PCI window");

static uint32_t phys_addr = 0;

static struct resource mcb_res = 
{
    .start  = 0,
    .end    = 0,
    .flags  = IORESOURCE_MEM,
    .name   = "mcb_window",
};

static struct uio_info mcb_platform_data = {
    .name = "MCB",
    .version = "1.0",
    .irq = -1,
};

static struct platform_device mcb_device = {
    .name   = "uio_pdrv_genirq",
    .id     = 0,
    .dev    = {
        .platform_data	= &mcb_platform_data,
    },
    .resource       = &mcb_res,
    .num_resources  = 1,
};

static int __init init_mcb(void)
{
    phys_addr = phys_addr_hi << 16;
    mcb_res.start = phys_addr;
    mcb_res.end = phys_addr + 64*1024*1024 - 1;
    mcb_res.flags = IORESOURCE_MEM;
    mcb_res.name = "mcb_window";
    
	printk(KERN_INFO "MCB-03PEM-PCI driver init, base address = %X\n", phys_addr);
	
    return platform_device_register(&mcb_device);
}

static void __exit cleanup_mcb(void)
{
	printk(KERN_INFO "MCB-03PEM-PCI shutdown\n");
	platform_device_unregister(&mcb_device);
}

module_init(init_mcb);
module_exit(cleanup_mcb);

MODULE_LICENSE("GPL");

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);

MODULE_SUPPORTED_DEVICE("MCB-03PEM-PCI");

